
// SimObject.m
//
// Base class for all object components of the VRRR project

#include "SimObject.h"
#import <math.h>
#include "globals.h"

@implementation SimObject



//////////////////////////////////////////
//
// createBegin
//
//////////////////////////////////////////
+ createBegin: aZone
{
  SimObject *obj = [super createBegin: aZone];
  
  obj->calculation= [Vector create: aZone];
  obj->oldPosition= [Vector create: aZone];

  obj->position   = [Vector create: aZone];
  obj->velocity   = [Vector create: aZone];
  obj->radius     = 1.0;
  
  obj->canvasItem    = nil;

  obj->model = aZone;

  return obj;
}



////////////////////////////////////////////
//
// createEnd
//
////////////////////////////////////////////
- createEnd
{
  return [super createEnd];
}


/////////////////////////////////////////
//
// drop
//
/////////////////////////////////////////
- (void) drop
{
  [position     drop];
  [velocity     drop];
  [calculation  drop];
  [oldPosition  drop];
  [super drop];
}

- init: (int) ident Type: (int) type
{
  objectID   = ident;
  objectType = type;
  return self;
}

- init: (int) ident Type: (int) type
      Position: (id <Vector>) p
      Velocity: (id <Vector>) v
      Radius: (double  ) r
{
  objectID   = ident;
  objectType = type;
  [position init: p];
  //  fprintf(stderr,"init vector %f %f\n",[position getX],[position getY]);
  [velocity init: v];
  radius = r;
  return self;
}


///////////////////////////////////////////
//
// initModel
//
//////////////////////////////////////////
- initModel: m
{
  model = m;
  worldX = [model getWorldXSize];
  worldY = [model getWorldYSize];

  return self;
}


//////////////////////////////////////////
//
// createCanvasItem
//
//////////////////////////////////////////
- createCanvasDisplayItem 
{
  canvasItem  = [Polygon createBegin: [self getZone]];

  if (model) 
  {

    [canvasItem setCanvas: [model getBoidCanvas]];
  }
  else 
  {
    [InternalError raiseEvent: "ERROR: SimObject >>>> createCanvasItem Model is not set\n"];
  }
  
  canvasItem = [canvasItem createEnd];
  return self;
}


- (id <Vector>)getVelocity 
{
  return velocity;
}

- (id <Vector>)getPosition 
{
  return position;
}

- (double)getRadius 
{
  return radius;
}

- step
{ // movement: p=p0 + v * dt. dt == 1
  [position add: velocity];
  //  fprintf(stderr, "Stepping %f %f ", [position getX],[position getY]);
  return self;
}

- setVelocity: (id <Vector>)v 
{
  [velocity init: v];
  return self;
}

- setPosition: (id <Vector>)v 
{
  [position init: v];
  return self;
}

- setRadius: (double) r 
{
  radius = r;
  return self;
}



//////////////////////////////////////////////////////////
//
// avoid
//
//////////////////////////////////////////////////////////
- (double) avoid: (id <Vector>)pos with: (id <Vector>)aVector
{
  double l;
  [aVector init: pos];
  [aVector sub: position];
  
  l = [aVector getLength] - radius;
  
  if (l < 0.0) {
    [aVector mult: 0.001]; // in object == very close
    l = 0.001;
  } else
    [aVector setLength: l];

  return l;
}

- (int)getObjectType 
{
  return objectType; 
}

- (int)getObjectID
{
  return objectID; 
}



/////////////////////////////////////////////////
//
//  canvasItem
//  This is the base class, and objects that move will override this method.
//  So it puts down only a stationary object
//
////////////////////////////////////////////////
- displayOnCanvas
{
 

  if( (![oldPosition eq: position]) && (canvasItem != nil) )  // need to draw
    { 

      [oldPosition init: position];    // update oldPosition

      [canvasItem erase];                 // erase the previous image

      //A stationary object is drawn
      //Formerly : case OBSTACLE:
	
      //
      // This should happen only once for each non-moving obstacle
      //
      { 
	double d;
	for (d=0.0; d<2*M_PI; d+=M_PI/10.0) 
	  {
	    id <Vector> v=[Vector create: [self getZone]];
	    [[v init: sin(d)*radius : cos(d)*radius] add: position];
	    [canvasItem add: v];
	  }
      }
	
      [canvasItem createItem];
    }

  return self;
}


////////////////////////////////////////////////
//
// drawSelfOn
//
// Draw an ellipse for the base type object
//
////////////////////////////////////////////////
- drawSelfOn: aRaster 
{


  // pj 2005-01-22: if you just want a point, use this
  //   [aRaster drawPointX: (unsigned) ([position getX])
  //   	   Y: (unsigned) ([position getY])
  //   	   Color: 0];

  [aRaster ellipseX0: (unsigned) [position getX]
	   Y0: (unsigned) [position getY]
	   X1: (unsigned) [position getX] + 8
	   Y1: (unsigned) [position getY] + 8
	   Width:  1
	   Color:  0];

  return self;
}


@end
